# -*- coding: utf-8 -*-
"""
Implementation of the Literal handling

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
@version: $Id: Literal.py,v 1.9 2007/10/04 13:02:10 ivan Exp $
$Date: 2007/10/04 13:02:10 $
"""

import re
from rdflib.Literal import Literal, _XSD_NS
from rdflib.RDF		import RDFNS  as ns_rdf
from rdflib.RDF     import XMLLiteral

#### helper functions
def _getLiteral(Pnode):
	"""Get (recursively) the full text from a DOM Node"""
	rc = ""
	for node in Pnode.childNodes:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
		elif node.nodeType == node.ELEMENT_NODE :
			rc = rc + _getLiteral(node)

	# This is still a question to me: do we really have to do such a normalization on the
	# text? Fo the time being, this is the resolution of the group, so the simple return
	# is changed to a normalized version
	# This is the 'simple', verbatim version
	# return rc
	# This is the normalized version
	return re.sub(r'(\r| |\n|\t)+'," ",rc).strip()

def _getXMLStructure(Pnode) :
	"""Get the XML Literal content of a DOM Node"""
	rc = ""
	for node in Pnode.childNodes:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
		elif node.nodeType == node.ELEMENT_NODE :
			rc = rc + node.toxml()
	return rc

#### the real meat...
def generateLiteral(node,graph,subject,state) :
	"""Generate the literal for the specific property, taking into account datatype, etc.
	Note: this method is called only if the @property is indeed present, no need to check and
	the @content property is also forgotten, because treated elsewhere...

	This should be the encoding of the algorithm well documented already on the wiki page:
	http://www.w3.org/2006/07/SWD/wiki/RDFa/LiteralObject

	@param node: DOM element node
	@param graph: the (RDF) graph to add the properies to
	"""
	assert node.hasAttribute("property")
	# Get the Property URI-s
	props = state.getResources(node.getAttribute("property"))

	# Get, if exists, the value of @datatype
	datatype = None
	dtset    = False
	if node.hasAttribute("datatype") :
		dtset = True
		dt = node.getAttribute("datatype")
		if dt != "" :
			datatype = state.getResource(dt)

	# The simple case: separate @content attribute
	if node.hasAttribute("content") :
		val = node.getAttribute("content")
		object = Literal(node.getAttribute("content"),datatype=datatype,lang=state.lang)
		# The value of datatype has been set, and the keyword paramaters take care of the rest
	else :
		# see if there *is* a datatype (even if it is empty!)
		if dtset :
			# yep. The Literal content is the pure text part of the current element:
			object = Literal(_getLiteral(node),datatype=datatype,lang=state.lang)
		else :
			# no controlling @datatype. We have to see if there is markup in the contained
			# element
			if True in [ n.nodeType == node.ELEMENT_NODE for n in node.childNodes ] :
				# yep, and XML Literal should be generated
				object = Literal(_getXMLStructure(node),datatype=XMLLiteral)
			else :
				object = Literal(_getLiteral(node),lang=state.lang)

	# The object may be empty, for example in an ill-defined <meta> element...
	if object != "" :
		for prop in props :
			graph.add((subject,prop,object))




###################################################################################
#
# $Log: Literal.py,v $
# Revision 1.9  2007/10/04 13:02:10  ivan
# *** empty log message ***
#
# Revision 1.8  2007/09/28 08:10:16  ivan
# *** empty log message ***
#
# Revision 1.7  2007/09/04 12:03:34  ivan
# *** empty log message ***
#
# Revision 1.6  2007/08/09 15:49:48  ivan
# Additional features built in to accomodate eRDF and DCMI style features, like:
#
# <meta> accepts @name instead of @property
# a prefix.value type rdf-qname is also accepted, not only prefix:value
# <link rel="schema.prefix"... type of elements are also accepted to define
# (top level) namespaces for the prefix
#
# Revision 1.5  2007/08/06 12:44:23  ivan
# After the first round of official tests. The following tests pass:
# 0001, 0006, 0007, 0008, 0009, 0010, 0011, 0012, 0013, 0014, 0018,
# 0029, 0030, 0031, 0032
#
#


