# -*- coding: utf-8 -*-
"""
The core Parsing function of RDFa. It follows the current specification. Some of the details are
put into other modules to make it clearer to update/modify.

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
@version: $Id: Parse.py,v 1.17 2007/10/04 13:02:10 ivan Exp $
$Date: 2007/10/04 13:02:10 $
"""

import sys

from pyRdfa.State   import State
from pyRdfa.Literal import generateLiteral

from rdflib.URIRef  import URIRef
from rdflib.BNode   import BNode
from rdflib.RDF     import RDFNS  as ns_rdf
from rdflib.RDF     import List, first, rest, nil, type, Seq, Bag, Alt

#######################################################################
# Function to check whether one of a series of attributes
# is part of the DOM Node
def _hasOneOfAttributes(node,*args) :
	"""
	Check whether one of the listed attributes is present on a node.
	@param node: DOM element node
	@param args: tuple consisting of possible attributes
	"""
	return True in [ node.hasAttribute(attr) for attr in args ]


#######################################################################
def parseOneNode(node,graph,currentResource,incomingState) :
	"""The (recursive) step of handling a single node when running down
	the DOM tree.

	@param node: the DOM node to handle
	@param graph: the RDF graph
	@type graph: RDFLib's Graph object instance
	@param currentResource: the current subject, as an RDFLib URIRef
	@param incomingState: the inherited state (namespaces, lang, etc)
	@type incomingState: pyRdfa.State instance
	"""
	chaining = False
	recurse  = True

	# Update the state. This means that the possible local settings of
	# namespaces and lang are set
	state = State(node,graph,inheritedState=incomingState)

	#---------------------------------------------------------------------------------

	# First, let us check whether there is anything to do at all. Ie,
	# whether there is any RDFa specific attribute on the element
	if not _hasOneOfAttributes(node,"href","resource","about","property","instanceof","rel","rev") :
		# nop, there is nothing to do here, just go down the tree...
		for n in node.childNodes :
			if n.nodeType == node.ELEMENT_NODE :
				parseOneNode(n,graph,currentResource,state)
		return
	#
	# there *is* something to do:-)
	# 1. Overriding the inherited subject in case an @object is present
	if node.hasAttribute("about") :
		currentResource = state.getCurieRef(node.getAttribute("about"))

	# -------------------------------------------------------------------------------
	# 2. Handling @property.
	# A particularity if property is that it usually stops the parsing down the DOM tree,
	# because everything down there is part of the generated literal. The exception is if
	# the property value is specified via an explicit @content
	if node.hasAttribute("property") :
		# Generate the literal. Put it into a separate module to make it more managable
		generateLiteral(node,graph,currentResource,state)
		if not node.hasAttribute("content") : recurse = False


	# --------------------------------------------------------------------------------
	# 3. See if a new subject will have to be generated for the containing elements.
	# The algorithm is, essentially, the one described by Ben in
	# http://www.w3.org/mid/46A8D3ED.2080404@adida.net
	# I have also added @data for <object>, can be taken out easily...
	# This branch takes also care of the @rel and @rev attributes
	if _hasOneOfAttributes(node,"rel","rev","resource","href","instanceof","about") :
		# These are the cases when a new subject has to be created!
		currentObjectResource = None

		# Note that CURIE can be used with @resource...
		if node.hasAttribute("resource") :
			currentObjectResource = state.getCurieRef(node.getAttribute("resource"))
		elif node.hasAttribute("href") :
			currentObjectResource = state.getURIRef(node.getAttribute("href"))
		elif node.hasAttribute("src") :
			currentObjectResource = state.getURIRef(node.getAttribute("src"))
		# As a fallback, a blank node should be generated...
		if not currentObjectResource :
			currentObjectResource = BNode()

		if node.hasAttribute("about") :
			currentElementIdentifier = state.getCurieRef(node.getAttribute("about"))
			instanceChains = False
		else :
			currentElementIdentifier = currentObjectResource
			instanceChains = True

		# Handle the @rel and @rev triples
		# both are definitely rdfa-qnames
		if node.hasAttribute("rel") :
			for prop in state.getResources(node.getAttribute("rel")) :
				chaining = True
				graph.add((currentResource,prop,currentObjectResource))
		if node.hasAttribute("rev") :
			for prop in state.getResources(node.getAttribute("rev")) :
				chaining = True
				graph.add((currentObjectResource,prop,currentResource))

		# Handling of instanceof...
		if node.hasAttribute("instanceof") :
			for r in state.getResources(node.getAttribute("instanceof")) :
				if instanceChains : chaining = True
				graph.add((currentElementIdentifier,type,r))

		# Here comes the essence of this part: the subject for descendents will be the newly generated resource...
		if chaining : currentResource = currentObjectResource

	# -------------------------------------------------------------------------------
	# Everything is settled for the current node. Go through the children, if necessary
	# For most of the cases, the index attribute is unnecessary, except for <li>-s.
	# It is easier to add it everywhere (and get it ignored down the line) rather
	# than finding out exactly those cases when it is in use...
	if recurse :
		for n in node.childNodes :
			if n.nodeType == node.ELEMENT_NODE :
				parseOneNode(n,graph,currentResource,state)

	# -------------------------------------------------------------------
	# This should be it...
	# -------------------------------------------------------------------


###################################################################################
#
# $Log: Parse.py,v $
# Revision 1.17  2007/10/04 13:02:10  ivan
# *** empty log message ***
#
# Revision 1.16  2007/09/28 08:10:16  ivan
# *** empty log message ***
#
# Revision 1.15  2007/09/12 15:25:53  ivan
# 1. adapted to the latest version of the processing step discussion (around @instanceof) though this version is not yet fully accepted by the group
# 2. Using now CURIE-s for @resource and @about
#
# Revision 1.14  2007/09/09 11:52:39  ivan
# The precedence of @src and @href was incorrect, compared to the specification...
#
# Revision 1.13  2007/09/04 12:03:34  ivan
# *** empty log message ***
#
# Revision 1.12  2007/08/22 11:11:58  ivan
# Moved the collection/container management into a separate module as a transformer.
# This meant adding the post-processing possibility, too
#
# Revision 1.11  2007/08/21 15:33:47  ivan
# Added the transformation mechanism. This means:
# - removed all 'special' cases from Parse and State (eg, handling of a meta element for @name,
# all traces of the 'dotted' qualified name notation from State
# - added the import and the management of transformer functions in __init__
#
# Revision 1.10  2007/08/09 15:49:49  ivan
# Additional features built in to accomodate eRDF and DCMI style features, like:
#
# <meta> accepts @name instead of @property
# a prefix.value type rdf-qname is also accepted, not only prefix:value
# <link rel="schema.prefix"... type of elements are also accepted to define
# (top level) namespaces for the prefix
#
# Revision 1.9  2007/08/08 10:59:24  ivan
# Added the front-end to handle URI-s (as opposed to file names only)
#
# Revision 1.8  2007/08/07 14:34:07  ivan
# Changed the core algorithm, see http://www.w3.org/2007/08/RdfaAlgo.{svg,png}
#
# Revision 1.7  2007/08/06 12:44:23  ivan
# After the first round of official tests. The following tests pass:
# 0001, 0006, 0007, 0008, 0009, 0010, 0011, 0012, 0013, 0014, 0018,
# 0029, 0030, 0031, 0032
#
#


