# -*- coding: utf-8 -*-
"""
Parser's state object and handling. The state, at present, includes:

  - dictionary for namespaces. Keys are the namespace prefixes, values are
RDFLib Namespace instances
  - language, retrieved from either @xml:lang or @lang
  - URI base, determined by @xml:base (or set explicitly)

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
@version: $Id: State.py,v 1.22 2007/10/04 13:02:10 ivan Exp $
$Date: 2007/10/04 13:02:10 $

"""
from rdflib.RDF         import RDFNS  as ns_rdf
from rdflib.Namespace   import Namespace
from rdflib.URIRef      import URIRef
from rdflib.Literal     import _XSD_NS
from rdflib.BNode       import BNode
import re

####Managing URIs
# This is the regex to decide whether a string is an absolute or relative URI...
# they come from http://www.gbiv.com/protocols/uri/rfc/rfc3986.html
_URIreg    = re.compile("[^:/?#]+:(.*)")
_URINoPath = re.compile("^(([^:/?#]+):)?(//([^/?#]*))?")


def _getPname(u) :
	"""Get the 'parent' name out of the URI, ie, the part of the URI
	that is one step 'up' in the path. That is the URI that should be
	used for local relative references
	"""
	# First we have to get rid of the possible fragment and query parts of the
	# URI (I guess:-(
	# 1. get the fragment part thrown out
	frag = u.rsplit('#',1)[0]
	# 2. get the query part thrown out
	path = frag.rsplit('?',1)[0]
	# 3. if the result is a 'slash URI', ie, if the
	# end is simply a '/' character, then the pname is simply that without the '/':
	if path[-1] == '/' :
		return path[0:-1]
	# 4. if the result is an absolute URI, then again the result should be that
	#
	mobj = _URINoPath.match(path)
	if mobj != None and mobj.group() == path :
		# we still have to see whether the *whole* string matched or not...
		return path

	# 5. Get the last portion of the path out
	paths = path.rsplit('/',1)
	if len(paths) > 1 :
		pname = paths[0]
	else :
		# this is the case when it is a local file name only...
		pname = "."
	return pname

#### Managing blank nodes for CURIE-s
_bnodes = {}
def _getBnodeFromCurie(var) :
	"""
	'Var' gives the string after the coloumn in a CURIE. If this variable has been used
	before, then the corresponding BNode is returned; otherwise a new BNode is created
	associated to that value.
	@param var: CURIE BNode identifier
	@return: BNode
	"""
	if len(var) == 0 :
		raise Exception("Illegal CURIE BNode identifier (empty local part)")
	if var in _bnodes :
		return _bnodes[var]
	else :
		retval = BNode()
		_bnodes[var] = retval
		return retval

#### Core Class definition
class State :
	"""State at a specific node, including the current set
	of namespaces in the RDFLib sense, the
	current language, and the base.
	"""
	def __init__(self,node,graph,inheritedState=None,base="") :
		"""
		@param node: the current DOM Node
		@param graph: the RDFLib Graph
		@param inheritedState: the state coming 'in' to node, as inherited
		from upper layers. This inheritedState is mixed with the state information
		retrieved from the current node
		@type inheritedState: State
		@param base: string denoting the base URI for the specific node. This is
		overridden by a possible @xml:base, but it overrides the possible
		base inherited from the upper layers. Note: at the moment (2007-08-11) it is not clear
		whether @xml:base processing will become part of RDFa or not. For the time being
		@xml:base handling is commented out and the <base> element in the header
		is used. (but setting the base at the top level
		is still of importance)
		"""
		# settling the base
		# note that, strictly speaking, it is not necessary to add the base to the
		# context, because there is only one place to set it (<base> element of the <header>).
		# This structure is slightly more complex because it is prepared for a possible
		# future change in direction of accepting xml:base on each element.

		if inheritedState :
			self.base  = inheritedState.base
			self.pname = inheritedState.pname # 'parent' name to handle relative uri-s
		else :
			self.base = ""
			for bases in node.getElementsByTagName("base") :
				if bases.hasAttribute("href") :
					self.base = bases.getAttribute("href")
					continue
			if self.base == "" :
				self.base = base
			self.pname = _getPname(base)

		# check first the lang or xml:lang attribute
		if node.hasAttribute("xml:lang") :
			self.lang = node.getAttribute("xml:lang")
			if len(self.lang) == 0 : self.lang = None
##      # It was agreed that @lang is not accepted for XHTML1+RDFa
##		elif node.hasAttribute("lang") :
##			self.lang = node.getAttribute("lang")
##			if len(self.lang) == 0 : self.lang = None
		elif inheritedState :
			self.lang = inheritedState.lang
		else :
			self.lang = None

		# see the namespaces stuff.
		dict = {}
		for i in range(0,node.attributes.length) :
			attr = node.attributes.item(i)
			if attr.prefix == "xmlns" :
				# yep, there is a namespace setting
				key = attr.localName
				if key != "" :
					# exclude the top level xmlns setting...
					uri = attr.value
					# 1. create a new Namespace entry
					ns = Namespace(uri)
					# 2. 'bind' it in the current graph to
					# get a nicer output
					graph.bind(key,uri)
					# 3. Add an entry to the dictionary
					dict[key] = ns

		# See if anything has been collected at all.
		# If not, the namespaces of the incoming state is
		# taken over
		self.ns = {}
		if len(dict) == 0 and inheritedState :
			self.ns = inheritedState.ns
		else :
			if inheritedState :
				for k in inheritedState.ns : self.ns[k] = inheritedState.ns[k]
				# copying the newly found namespace, possibly overwriting
				# incoming values
				for k in dict :  self.ns[k] = dict[k]
			else :
				self.ns = dict
		# extra trick for unusual usages...
		# if the 'rdf' prefix is not used, it is artificially added...
		if "rdf" not in self.ns :
			self.ns["rdf"] = ns_rdf
		# this is just convenience; in reality, the user should set this
		# thing...
		if "xsd" not in self.ns :
			self.ns["xsd"] = _XSD_NS

	def getResource(self,val) :
		"""The input argument is a rdfa-QNAME; this is interpreted
		via the current namespaces and the corresponding ref is returned
		@param val: string of the form prefix':'lname
		@return: an RDFLib URIRef instance
		"""
		if val == "" :
			return None
		elif val.find(":") != -1 :
			key   = val.split(":",1)[0]
			lname = val.split(":",1)[1]
			# A small error: this method is invoked for property URI-s, which
			# should not refer to a blank node. This case is checked and a possible
			# error condition is handled
			if key == "_" :
				raise Exception("Blank node CURIE cannot be used in this position: _:%s" % lname)
		else :
			return None

		if key not in self.ns :
			return None
		else :
			return self.ns[key][lname]

	def getResources(self,val) :
		"""The input argument is a list of rdfa-QNAMEs; this is interpreted
		via the current namespaces and the corresponding refs are returned
		@param val: strings of the form prefix':'lname, separated by space
		@return: a list of RDFLib URIRef instances
		"""
		val.strip()
		resources = [ self.getResource(v) for v in val.split() if v != None ]
		return [ r for r in resources if r != None ]

	def getURIRef(self,val) :
		"""The input argument is a URI and is checked whether it is a local
		reference with a '#' or not. In the former case a URIRef combined with the
		stored base value. In both cases a URIRef based on the full URI is created
		and returned
		@param val: URI string
		@return: an RDFLib URIRef instance
		"""
		if val == "" :
			return URIRef(self.base)
		elif val[0] == '#' :
			return URIRef(self.base + val)
		elif _URIreg.match(val.strip()) != None :
			# this is an absolute URI
			return URIRef(val)
		else :
			# this is a relative URI that has to be combined with the base pname value
			if self.pname == "." :
				return URIRef(val)
			else :
				return URIRef(self.pname + "/" + val)

	def getCurieRef(self,val) :
		"""The input argument is a CURIE. This means that it is
		  - either of the form [a:b] where a:b should be resolved as an rdfa-QNAME, or
		  - it is a traditional URI (relative or absolute)

		@param val: CURIE string
		@return: an RDFLib URIRef instance
		"""
		if len(val) == 0 :
			return URIRef(self.base)
		elif val[0] == "[" :
			if val[-1] == "]" :
				curie = val[1:-1]
				# A possible Blank node reference should be separated here:
				if len(curie) >= 2 and curie[0] == "_" and curie[1] == ":" :
					return _getBnodeFromCurie(curie[2:])
				else :
					return self.getResource(val[1:-1])
			else :
				# illegal CURIE...
				raise Exception("Illegal CURIE: %s" % val)
		else :
			return self.getURIRef(val)

####################################################################################
#
# $Log: State.py,v $
# Revision 1.22  2007/10/04 13:02:10  ivan
# *** empty log message ***
#
# Revision 1.21  2007/09/28 08:10:16  ivan
# *** empty log message ***
#
# Revision 1.20  2007/09/24 16:01:08  ivan
# *** empty log message ***
#
# Revision 1.19  2007/09/24 15:03:30  ivan
# *** empty log message ***
#
# Revision 1.18  2007/09/13 15:24:02  ivan
# empty @lang should generate a no language...
#
# Revision 1.17  2007/09/13 14:16:40  ivan
# Added the [_:xx] syntax handling for blank nodes
#
# Revision 1.16  2007/09/12 15:26:38  ivan
# 1. Renamed the method to create URI resources
# 2. Added the facility to retrieve CURIE via "[...]" syntax
#
# Revision 1.15  2007/08/21 15:33:47  ivan
# Added the transformation mechanism. This means:
# - removed all 'special' cases from Parse and State (eg, handling of a meta element for @name,
# all traces of the 'dotted' qualified name notation from State
# - added the import and the management of transformer functions in __init__
#
# Revision 1.14  2007/08/12 08:38:33  ivan
# Added the list of predefined property and rel/rev values (though not yes used
# in the code)
#
# Revision 1.13  2007/08/11 13:29:39  ivan
# Took the @xml:base processing out; added processing of <base>.
#
# Revision 1.12  2007/08/11 07:42:15  ivan
# Removed @xml:base handling
# Also settled an extra thing with the dotted issue: if there *is* a dot
# in the property name, that one is ignored...
#
# Revision 1.11  2007/08/11 07:15:19  ivan
# Commented out much of the 'dotted' property handling in the <head>, to leave it
# to the Task Force to decide on the final form. The only think that is still there
# is the general form of handling the non-namespaced property names instead of
# restricting this to a set of 'known' values.
#
# Revision 1.10  2007/08/10 09:46:28  ivan
# Adjusted the backward compatibility issues:
# - dotted notation allowed in the <head> only, including the dotted notation for
# xmlns setting
# - xmlns namespace added to property names in the <link>/<meta> elements only.
#
# Revision 1.9  2007/08/09 15:49:49  ivan
# Additional features built in to accomodate eRDF and DCMI style features, like:
#
# <meta> accepts @name instead of @property
# a prefix.value type rdf-qname is also accepted, not only prefix:value
# <link rel="schema.prefix"... type of elements are also accepted to define
# (top level) namespaces for the prefix
#
# Revision 1.8  2007/08/07 14:34:07  ivan
# Changed the core algorithm, see http://www.w3.org/2007/08/RdfaAlgo.{svg,png}
#
# Revision 1.6  2007/08/06 12:44:23  ivan
# After the first round of official tests. The following tests pass:
# 0001, 0006, 0007, 0008, 0009, 0010, 0011, 0012, 0013, 0014, 0018,
# 0029, 0030, 0031, 0032
#
# Revision 1.7  2007/08/07 09:26:29 ivan  ivan
# significantly extended the _getPname (formerly _getLname) function to
# include different type of URI (domain only, URIs with query and fraction parts, etc)
#
#

