# -*- coding: utf-8 -*-
"""
RDFa parser, reflecting the current Working Group discussions.

See http://dev.w3.org/2004/PythonLib-IH/pyRdfa/

Typical usage::
 from pyRdfa import processFile
 print processFile('filename.html')

Internal structure
------------------

The package uses the concept of 'transformers': the parsed DOM tree is possibly
transformed before performing the real RDFa processing. Optionally, the resulting
RDF Graph is also post-processed. This transformer structure makes it possible to
add additional 'services' without distoring the core code of RDFa processing.

Some of these transformations are always performed. At this time, these are:

 - putting a number of predefined attributes and properties into the XHTML namespace (eg, next, alternate, ...)
 - 'ol' and 'ul' elements are possibly transformed to generate collections or containers. The former also requires a post-processing step

Some of the transformations are optional; by default, they are processed, but can be switched off. These are:

 - the 'name' attribute of the 'meta' element is copied into a 'property' attribute of the same element
 - interpreting the 'openid' references in the header
 - implementing the Dublin Core dialect to include DC statements from the header

Finally, the caller of the package may also add his/her transformer modules.

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
@version: $Id: __init__.py,v 1.16 2007/10/04 13:02:34 ivan Exp $
$Date: 2007/10/04 13:02:34 $
"""
__version__ = "0.6"

import sys

from rdflib.Graph       import Graph
from rdflib.URIRef      import URIRef
from rdflib.RDF			import RDFNS  as ns_rdf
from rdflib.RDFS		import RDFSNS as ns_rdfs

from pyRdfa.State import State, _URIreg
from pyRdfa.Parse import parseOneNode

from xml.dom.minidom import parse

from pyRdfa.transform.utils import dump

from pyRdfa.transform.MetaName              import MetaTransform
from pyRdfa.transform.HeadAbout             import HeadAboutTransform
from pyRdfa.transform.PredefinedProps       import PredefinedAttributeTransform
from pyRdfa.transform.OpenID                import OpenIDTransform
from pyRdfa.transform.DublinCore            import DCTransform
from pyRdfa.transform.ContainersCollections import decorateLIs

# Exception handling. Essentially, all the different exceptions are re-packaged into
# separate exception class, to allow for an easier management on the user level

class RDFaError(Exception) :
	pass


# List of built-in transformers that are to be run regardless, because
# they are part of the RDFa spec
builtInTransformers = [
	PredefinedAttributeTransform,
	HeadAboutTransform,
	decorateLIs
]

# List of optional built-in transformers that are run if the caller sets
# the appropriate flag
extraTransformers = [
	MetaTransform,
	OpenIDTransform,
	DCTransform
]


def _process(input,base,outputFormat,transformers,postprocessors,extras) :
	"""The real processing
	@param input: file like object for the XHTML input
	@param outputFormat: serialization format
	@param transformers: list of transformation methods to pre-process the DOM tree before
	parsing it for RDFa
	@rtype transformer: list
	@param postprocessors: list of postprocessing steps performed on the generated RDF graph
	@rtype postprocessors: list
	@param builtins: whether the built-in transformers should be invoked or not
	@rtype builtins: boolean
	@return: serialized graph
	@rtype: string
	"""
	# The current turtle serializer makes very funny things. So I artificially switch the
	# turtle serializer to n3, which seems to be better. Maybe in future this becomes unnecessary...
	if outputFormat == "turtle" : outputFormat = "n3"

	# Create the RDF Graph
	graph   = Graph()
	# get the DOM tree
	try :
		dom  	= parse(input)
	except :
		# XML Parsing error in the input
		(type,value,traceback) = sys.exc_info()
		msg = 'Parsing error in input file: "%s"' % value
		raise RDFaError, msg

	html 	= dom.documentElement

	# Perform the built-in and external transformations on the HTML tree. This is,
	# in simulated form, the hGRDDL approach of Ben Adida
	if extras:
		transfuncs = transformers + extraTransformers + builtInTransformers
	else :
		transfuncs = transformers + builtInTransformers
	for trans in transfuncs : trans(html)

	# collect the initial state. This takes care of things
	# like base, top level namespace settings, etc.
	state   = State(html,graph,base=base)

	# The top level subject starts with the current document; this
	# is used by the recursion
	subject = URIRef(state.base)

	# parse the whole thing recursively and fill the graph
	try :
		# this is a recursive procedure through the full DOM Tree
		parseOneNode(html,graph,subject,state)
	except :
		# error in the input...
		(type,value,traceback) = sys.exc_info()
		msg = 'Error in RDFa content: "%s"' % value
		raise RDFaError, msg

	# serialize the graph and return the result
	retval = None
	try :
		retval = graph.serialize(format=outputFormat)
	except :
		# XML Parsing error in the input
		(type,value,traceback) = sys.exc_info()
		msg = 'Error in graph serialization: "%s"' % value
		raise RDFaError, msg
	return retval

def processURI(uri,outputFormat="xml",transformers=[],postprocessors=[],extras=True) :
	"""The standard processing of an RDFa file:
	@param input: URI to access
	@param outputFormat: serialization format, understood by RDFLib. Note that though "turtle" is
	a possible parameter value, the RDFLib turtle generation does funny (though legal) things with
	namespaces, defining unusual and unwanted prefixes...
	@param transformers: list of transformation methods to pre-process the DOM tree before
	parsing it for RDFa
	@rtype transformer: list
	@param postprocessors: list of postprocessing steps performed on the generated RDF graph
	@rtype postprocessors: list
	@param extras: whether the extra built-in transformers should be invoked or not
	@rtype extras: boolean
	@return: serialized graph
	@rtype: string
	"""
	import urllib
	# Note the that method below will raise exceptions if the
	# uri cannot be dereferenced properly...
	# It is also the simplest dereferencing. Ie, no handling of redirections, for example...
	input = None
	try :
		input = urllib.urlopen(uri)
	except :
		# XML Parsing error in the input
		(type,value,traceback) = sys.exc_info()
		msg = 'Problems in accessing the information: "%s"\n (problematic URI: %s)' % (value,uri)
		raise RDFaError, msg

	return _process(input,uri,outputFormat,transformers,postprocessors,extras)

def processFile(fileName,outputFormat="xml",transformers=[],postprocessors=[],extras=True) :
	"""The standard processing of an RDFa file:
	@param input: input file name
	@param outputFormat: serialization format, understood by RDFLib. Note that though "turtle" is
	a possible parameter value, the RDFLib turtle generation does funny (though legal) things with
	namespaces, defining unusual and unwanted prefixes...
	@param transformers: list of transformation methods to pre-process the DOM tree before
	parsing it for RDFa
	@rtype transformer: list
	@param postprocessors: list of postprocessing steps performed on the generated RDF graph
	@rtype postprocessors: list
	@param extras: whether the extra built-in transformers should be invoked or not
	@rtype extras: boolean
	@return: serialized graph
	@rtype: string
	"""
	input = None
	try :
		input = file(fileName)
	except :
		# XML Parsing error in the input
		(type,value,traceback) = sys.exc_info()
		msg = 'Problems in opening the file: "%s"\n (problematic file name: %s)' % (value,fileName)
		raise RDFaError, msg
	return _process(input,fileName,outputFormat,transformers,postprocessors,extras)

#
#
# $Log: __init__.py,v $
# Revision 1.16  2007/10/04 13:02:34  ivan
# *** empty log message ***
#
# Revision 1.15  2007/10/04 13:02:10  ivan
# *** empty log message ***
#
# Revision 1.14  2007/09/28 08:10:16  ivan
# *** empty log message ***
#
# Revision 1.13  2007/09/24 15:03:30  ivan
# *** empty log message ***
#
# Revision 1.12  2007/09/06 13:56:17  ivan
# Artificially switch the turtle output to n3
#
# Revision 1.11  2007/09/04 12:03:34  ivan
# *** empty log message ***
#
# Revision 1.10  2007/09/03 12:06:39  ivan
# The @name in the <meta> element transformer has been moved to the 'optional' category
#
# Revision 1.9  2007/08/22 11:11:58  ivan
# Moved the collection/container management into a separate module as a transformer.
# This meant adding the post-processing possibility, too
#
# Revision 1.8  2007/08/21 15:33:47  ivan
# Added the transformation mechanism. This means:
# - removed all 'special' cases from Parse and State (eg, handling of a meta element for @name,
# all traces of the 'dotted' qualified name notation from State
# - added the import and the management of transformer functions in __init__
#
# Revision 1.7  2007/08/08 10:59:24  ivan
# Added the front-end to handle URI-s (as opposed to file names only)
#
# Revision 1.6  2007/08/06 12:44:23  ivan
# After the first round of official tests. The following tests pass:
# 0001, 0006, 0007, 0008, 0009, 0010, 0011, 0012, 0013, 0014, 0018,
# 0029, 0030, 0031, 0032
#
#


