# -*- coding: utf-8 -*-
"""
Transfomer: handles the RDF collections and containers

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
@version: $Id: ContainersCollections.py,v 1.3 2007/10/04 13:02:46 ivan Exp $
$Date: 2007/10/04 13:02:46 $
"""

import random
from pyRdfa.Parse           import _hasOneOfAttributes
from pyRdfa.transform.utils import rdfPrefix, traverseTree, dump

_NONE       = 0
_CONTAINER  = 1
_COLLECTION = 2

_bids = []

def _collectBIDs(node) :
	"""Check and collect the possible bnode id-s in the file that might occur in CURIE-s. The
	function is called recursively on each node.
	@param node: a DOM element node
	"""
	def _suspect(val) :
		if len(val) > 1 :
			if val[0] == "_" and val[1] == ":" :
				if not val in _bids : _bids.append(val)
			elif val[0] == "[" and val[-1] == "]" :
				_suspect(val[1:-1])
	for a in ["about","resource","instanceof"] :
		if node.hasAttribute(a) : _suspect(node.getAttribute(a))
	return False

def _giveBID() :
	"""Generate a new value that can be used as a bnode id...
	@return: a string of the form _:XXXX where XXXX is unique
	"""
	while True :
		i = random.randint(1,10000)
		val = "_:x%s" % i
		if not val in _bids :
			_bids.append(val)
			return val

def _checkIfHit(node,rdfprefix) :
	if node.hasAttribute("instanceof") :
		types = node.getAttribute("instanceof").split()
		for t in types :
			# check if it is a namespaces thing at all...
			if t.find(":") != -1 :
				key   = t.split(":",1)[0]
				lname = t.split(":",1)[1]
				if key == rdfprefix :
					if lname in ["Seq","Alt","Bag"] :
						return _CONTAINER
					elif lname in ["List"] :
						return _COLLECTION
	return _NONE

def _decorateContainer(node,rdfprefix) :
	"""Take care of containers (Seq,Alt,Bag).
	@param node: the node for the <ul>/<ol>
	@param rdfprefix: the prefix of the RDF namespace
	"""
	index = 1
	for n in node.childNodes :
		if n.nodeType == node.ELEMENT_NODE and n.tagName == "li" :
			# set the extra property/rel to be added
			pr = "%s:_%s"  % (rdfprefix,index)
			if _hasOneOfAttributes(n,"href","resource","instanceof","about") :
				n.setAttribute("rel",pr)
			else :
				n.setAttribute("property",pr)
			index += 1

def _decorateCollection(node,rdfprefix) :
	"""Take care of containers (Seq,Alt,Bag).
	@param node: the node for the <ul>/<ol>
	@param rdfprefix: the prefix of the RDF namespace
	"""
	List  = "%s:List"  % rdfprefix
	first = "%s:first" % rdfprefix
	rest  = "%s:next"  % rdfprefix
	nil   = "%s:nil"   % rdfprefix
	rtype = "%s:type"  % rdfprefix
	# the list of 'li'-s is needed in advance (eg, for their numbers)
	lis = [ n for n in node.childNodes if n.nodeType == node.ELEMENT_NODE and n.tagName == "li" ]
	# a bnode id should be generated for the top level node
	currId = "[%s]" % _giveBID()
	node.setAttribute("resource",currId)
	index = 1
	for i in xrange(0,len(lis)) :
		n = lis[i]
		# first the current <li> must be massaged
		n.setAttribute("about",currId)
		if _hasOneOfAttributes(n,"href","resource","instanceof") :
			n.setAttribute("rel",first)
			# The instanceof thingy has to move:-(
			if n.hasAttribute("instanceof") :
				for t in n.getAttribute("instanceof").strip().split() :
					span = n.ownerDocument.createElement("span")
					span.setAttribute("rel",rtype)
					span.setAttribute("resource","[%s]" % t)
					n.appendChild(span)
			n.setAttribute("instanceof","")
		else :
			n.setAttribute("property",first)

		newLi = node.ownerDocument.createElement("li")
		newLi.setAttribute("about",currId)
		newLi.setAttribute("rel",rest)
		if i == len(lis) - 1 :
			# This is the last element
			newLi.setAttribute("resource",nil)
			node.appendChild(newLi)
		else :
			newId = "[%s]" % _giveBID()
			newLi.setAttribute("resource",newId)
			newLi.setAttribute("instanceof",List)
			currId = newId
			node.insertBefore(newLi,lis[i+1])
	#dump(node)

def decorateLIs(html) :
	"""
	@param html: a DOM node with the top level html element
	@param state: top level state
	"""
	uls = [ n for n in html.getElementsByTagName("ul") ]
	ols = [ n for n in html.getElementsByTagName("ol") ]
	if len(uls) == 0 and len(ols) == 0 : return

	#--------------------------------------------------------------------------------
	# We have to extract the prefix used for rdf. It may not be 'rdf'...
	rdfprefix = rdfPrefix(html)

	#--------------------------------------------------------------------------------
	# We have to collect the current bnode id-s from the file to avoid conflicts
	traverseTree(html,_collectBIDs)
	# We will need the random function to generate unique bid-s
	random.seed(None)

	#--------------------------------------------------------------------------------

	for node in uls+ols :
		# check if this is one of those guys...
		t = _checkIfHit(node,rdfprefix)
		if t == _CONTAINER :
			_decorateContainer(node,rdfprefix)
		elif t == _COLLECTION :
			_decorateCollection(node,rdfprefix)

