import os, os.path
import xml.sax.saxutils
from unittest import TestCase, makeSuite, TextTestRunner
from re import search
from StringIO import StringIO
from urllib2 import urlopen

from pyRdfa import processURI
from rdflib import Namespace
from rdflib.Graph import Graph
from rdflib.sparql.bison.Processor import Processor

class XhtmlTests (TestCase):
  """
  This test suite is generated automatically from the rdfa test manifest at:
  
  http://www.w3.org/2006/07/SWD/RDFa/testsuite/xhtml1-testcases/

  When this file is compiled a sparql query is executed against the 
  manifest, and a unittest TestCase is generated.
  """

def write_review(num, sparql_value, xhtml_text, n3rdf_text, sparql_text,
                 fname):
  """
  Writes a review file, which is an XHTML file that contains the original
  XHTML markup, the generated triples in N3 format, the SPARQL query, and
  whether or not the test succeeded, or failed.
  """
  
  reviewfile = open(fname, "w")
  reviewhtml = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML+RDFa 1.0//EN" "http://www.w3.org/MarkUp/DTD/xhtml-rdfa-1.dtd">
<html>
  <head>
    <title>%s</title>    
  </head>
  <body>
  <h1>XHTML</h1>
  <pre>
%s
  </pre>
  <h1>Triples (N3 format)</h1>
  <pre>
%s
  </pre>
  <h1>SPARQL (%s)</h1>
  <pre>
%s
  </pre>
  </body>
</html>
  '''
  test_status = "PASSED"
  if(not sparql_value):
    test_status = "FAILED"
  title = "RDFa Test Suite #%04i - %s" % (num, test_status)
  reviewfile.write(reviewhtml % (title,
                                 xml.sax.saxutils.escape(xhtml_text),
                                 xml.sax.saxutils.escape(n3rdf_text),
                                 test_status,
                                 xml.sax.saxutils.escape(sparql_text)))
  reviewfile.close()

def make_test(html_uri, sparql_uri, title):
  """
  Given an html_uri for some rdfa content, a uri for a sparql query to 
  execute, and the title of the test this function will create
  the test for it.
  """
  def test(self):
    rdf = processURI(html_uri)
    graph = Graph()
    graph.parse(StringIO(rdf))
    sparql = urlopen(sparql_uri).read()
    sparql_processor = Processor(graph)
    sparql_value = sparql_processor.query(sparql)

    num = int(search(r'(\d+)\.xhtml', html_uri).groups(1)[0])
    basename = "%04i" % num
    if(not sparql_value):
      basename = "%04i-FAIL" % num

    # Write the XHTML to a file
    xhtml_text = urlopen(html_uri).read()
    xhtmlfile = open(os.path.join("review", "%s.xhtml" % (basename,)), "w")
    xhtmlfile.write(xhtml_text)
    xhtmlfile.close()

    # write the N3 output to a file
    n3rdf_text = processURI(html_uri, outputFormat="n3")
    n3file = open(os.path.join("review", "%s.n3" % (basename,)), "w")
    n3file.write(str(n3rdf_text))
    n3file.close()

    # Write the SPARQL to a file
    sparql_text = urlopen(sparql_uri).read()
    sparqlfile = open(os.path.join("review", "%s.sparql" % (basename,)), "w")
    sparqlfile.write(sparql_text)
    sparqlfile.close()

    # Write the review file
    write_review(num, sparql_value, xhtml_text, n3rdf_text, sparql_text,
                 os.path.join("review", "%s-review.html" % (basename,)))

    self.assertTrue(sparql_value, title)
  return test

# query the RDFa test manifest and generate test methods in the 
# XhtmlTests unittest

q = """
PREFIX test: <http://www.w3.org/2006/03/test-description#> 
PREFIX dc:   <http://purl.org/dc/elements/1.1/>
SELECT ?html_uri ?sparql_uri ?title ?status
FROM <http://www.w3.org/2006/07/SWD/RDFa/testsuite/xhtml1-testcases/rdfa-xhtml1-test-manifest.rdf>
WHERE 
{ 
  ?t dc:title ?title .
  ?t test:informationResourceInput ?html_uri .
  ?t test:informationResourceResults ?sparql_uri .
  ?t test:reviewStatus ?status .
}
"""

g = Graph()
for html, sparql, title, status_url in g.query(q):
  status = status_url.split("#")[-1]
  if(status == "approved" or status == "unreviewed"):
    num = search(r'(\d+)\.xhtml', html).groups(1)
    setattr(XhtmlTests, "test-%02i-%s" % (int(num[0]), status),
            make_test(html, sparql, title))

# when run from the command line this will execute each test
if __name__ == '__main__':
  # Remove all of the previous N3 files
  if(os.path.exists("review")):
    files = os.listdir("review")
    for f in files:
      os.unlink(os.path.join("review", f))
    os.rmdir('review')
  os.mkdir('review')
      
  suite = makeSuite(XhtmlTests)
  runner = TextTestRunner(descriptions=2, verbosity=2)
  runner.run(suite)
